@@ -4,10 +4,10 @@
 #' at each position. This is inspired by the SQL `COALESCE` function
 #' which does the same thing for `NULL`s.
 #'
-#' @param ... Vectors. All inputs should either be length 1, or the
-#'   same length as the first argument.
-#'
-#'   These dots support [tidy dots][rlang::tidy-dots] features.
+#' @param ... <[`dynamic-dots`][rlang::dyn-dots]> Vectors. Inputs should be
+#'   recyclable (either be length 1 or same length as the longest vector) and
+#'   coercible to a common type. If data frames, they are coalesced column by
+#'   column.
 #' @return A vector the same length as the first `...` argument with
 #'   missing values replaced by the first non-missing value.
 #' @seealso [na_if()] to replace specified values with a `NA`.
@@ -23,7 +23,7 @@
 #' z <- c(NA, NA, 3, 4, 5)
 #' coalesce(y, z)
 #'
-#' # Supply lists by splicing them into dots:
+#' # Supply lists by with dynamic dots
 #' vecs <- list(
 #'   c(1, 2, NA, NA, 5),
 #'   c(NA, NA, 3, 4, 5)
@@ -31,19 +31,39 @@
 #' coalesce(!!!vecs)
 coalesce <- function(...) {
   if (missing(..1)) {
-    abort("At least one argument must be supplied")
+    abort("At least one argument must be supplied.")
   }
 
   values <- list2(...)
+  values <- vec_cast_common(!!!values)
+  values <- vec_recycle_common(!!!values)
+
   x <- values[[1]]
   values <- values[-1]
 
-  for (i in seq_along(values)) {
-    x <- replace_with(
-      x, is.na(x), values[[i]],
-      glue("Argument {i + 1}"),
-      glue("length of {fmt_args(~x)}")
-    )
+  if (!is_null(attr(x, "dim"))) {
+    abort("Can't coalesce matrices or arrays.")
   }
+  if (is.data.frame(x)) {
+    df_coalesce(x, values)
+  } else {
+    vec_coalesce(x, values)
+  }
+}
+
+vec_coalesce <- function(x, values) {
+  for (i in seq_along(values)) {
+    x_miss <- is.na(x)
+    vec_slice(x, x_miss) <- vec_slice(values[[i]], x_miss)
+  }
+
+  x
+}
+df_coalesce <- function(x, values) {
+  for (i in seq_along(x)) {
+    col_values <- map(values, `[[`, i)
+    x[[i]] <- vec_coalesce(x[[i]], col_values)
+  }
+
   x
 }
