@@ -1,5 +1,6 @@
 #' Force computation of a database query
 #'
+#' @description
 #' `compute()` stores results in a remote temporary table.
 #' `collect()` retrieves data into a local tibble.
 #' `collapse()` is slightly different: it doesn't force computation, but
@@ -8,10 +9,21 @@
 #'
 #' All functions preserve grouping and ordering.
 #'
-#' @param x A tbl
+#' @section Methods:
+#' These functions are **generics**, which means that packages can provide
+#' implementations (methods) for other classes. See the documentation of
+#' individual methods for extra arguments and differences in behaviour.
+#'
+#' Methods available in currently loaded packages:
+#' * `compute()`: \Sexpr[stage=render,results=rd]{dplyr:::methods_rd("compute")}
+#' * `collect()`: \Sexpr[stage=render,results=rd]{dplyr:::methods_rd("collect")}
+#' * `collapse()`: \Sexpr[stage=render,results=rd]{dplyr:::methods_rd("collapse")}
+#'
+#' @param x A data frame, data frame extension (e.g. a tibble), or a lazy
+#'   data frame (e.g. from dbplyr or dtplyr). See *Methods*, below, for more
+#'   details.
 #' @param name Name of temporary table on database.
 #' @param ... Other arguments passed on to methods
-#' @inheritParams copy_to.src_sql
 #' @seealso [copy_to()], the opposite of `collect()`: it takes a local data
 #'   frame and uploads it to the remote source.
 #' @export
@@ -36,15 +48,20 @@
 compute <- function(x, name = random_table_name(), ...) {
   UseMethod("compute")
 }
-
 #' @export
 #' @rdname compute
 collect <- function(x, ...) {
   UseMethod("collect")
 }
-
 #' @export
 #' @rdname compute
 collapse <- function(x, ...) {
   UseMethod("collapse")
 }
+
+#' @export
+collect.data.frame <- function(x, ...) x
+#' @export
+compute.data.frame <- function(x, ...) x
+#' @export
+collapse.data.frame <- function(x, ...) x
