@@ -1,17 +1,19 @@
 #' Create a "src" object
 #'
-#' \code{src} is the standard constructor for srcs and \code{is.src} tests.
+#' `src()` is the standard constructor for srcs and `is.src()` tests.
 #'
 #' @keywords internal
 #' @export
 #' @param subclass name of subclass. "src" is an abstract base class, so you
-#'   must supply this value. \code{src_} is automatically prepended to the
+#'   must supply this value. `src_` is automatically prepended to the
 #'   class name
-#' @param ... fields used by object
+#' @param ... fields used by object.
+#'
+#'   These dots are evaluated with [explicit splicing][rlang::dots_list].
 #' @param x object to test for "src"-ness.
 src <- function(subclass, ...) {
   subclass <- paste0("src_", subclass)
-  structure(list(...), class = c(subclass, "src"))
+  structure(dots_list(...), class = c(subclass, "src"))
 }
 
 #' @rdname src
@@ -30,8 +32,10 @@ print.src <- function(x, ...) {
 #' possible results will be.
 #'
 #' @param x a data src.
+#' @param ... other arguments passed on to the individual methods.
 #' @export
-src_tbls <- function(x) {
+#' @keywords internal
+src_tbls <- function(x, ...) {
   UseMethod("src_tbls")
 }
 
@@ -44,3 +48,8 @@ src_tbls <- function(x) {
 same_src <- function(x, y) {
   UseMethod("same_src")
 }
+
+#' @export
+same_src.data.frame <- function(x, y) {
+  is.data.frame(y)
+}
