@@ -1,62 +1,24 @@
 context("colwise mutate/summarise")
 
-test_that("funs found in current environment", {
-  f <- function(x) 1
-  df <- data.frame(x = c(2:10, 1000))
-
-  with_lifecycle_silence({
-    out <- summarise_all(df, funs(f, mean, median))
-  })
-  expect_equal(out, data.frame(f = 1, mean = 105.4, median = 6.5))
-
-  out <- summarise_all(df, list(f = f, mean = mean, median = median))
-  expect_equal(out, data.frame(f = 1, mean = 105.4, median = 6.5))
-  # TODO: expect_error(summarise_all(df, list(f, mean, median)))
-})
-
-test_that("can use character vectors", {
+test_that("can use character vectors or bare functions", {
   df <- data.frame(x = 1:3)
-  scoped_lifecycle_silence()
+  expect_equal(summarise_all(df, "mean"), data.frame(x = 2))
+  expect_equal(summarise_all(df, mean), data.frame(x = 2))
 
-  expect_equal(summarise_all(df, "mean"), summarise_all(df, funs(mean)))
-  expect_equal(mutate_all(df, list(mean = "mean")), mutate_all(df, funs(mean = mean)))
-
-  expect_equal(summarise_all(df, "mean"), summarise_all(df, list(mean)))
-  expect_equal(mutate_all(df, list(mean = "mean")), mutate_all(df, list(mean = mean)))
-})
-
-test_that("can use bare functions", {
-  df <- data.frame(x = 1:3)
-  scoped_lifecycle_silence()
-
-  expect_equal(summarise_all(df, mean), summarise_all(df, funs(mean)))
-  expect_equal(mutate_all(df, mean), mutate_all(df, funs(mean)))
-
-  expect_equal(summarise_all(df, mean), summarise_all(df, list(mean)))
-  expect_equal(mutate_all(df, mean), mutate_all(df, list(mean)))
+  expect_equal(mutate_all(df, list(x = "mean")), data.frame(x = rep(2, 3)))
+  expect_equal(mutate_all(df, list(x = mean)), data.frame(x = rep(2, 3)))
 })
 
 test_that("default names are smallest unique set", {
   df <- data.frame(x = 1:3, y = 1:3)
-  scoped_lifecycle_silence()
-
-  expect_named(summarise_at(df, vars(x:y), funs(mean)), c("x", "y"))
-  expect_named(summarise_at(df, vars(x), funs(mean, sd)), c("mean", "sd"))
-  expect_named(summarise_at(df, vars(x:y), funs(mean, sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
-  expect_named(summarise_at(df, vars(x = x), funs(mean, sd)), c("x_mean", "x_sd"))
 
   expect_named(summarise_at(df, vars(x:y), list(mean)), c("x", "y"))
   expect_named(summarise_at(df, vars(x), list(mean = mean, sd = sd)), c("mean", "sd"))
   expect_named(summarise_at(df, vars(x:y), list(mean = mean, sd = sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
-
-  expect_named(summarise_at(df, vars(x:y), funs(base::mean, stats::sd)), c("x_base::mean", "y_base::mean", "x_stats::sd", "y_stats::sd"))
 })
 
-test_that("named arguments force complete named", {
-  scoped_lifecycle_silence()
+test_that("named arguments force complete names", {
   df <- data.frame(x = 1:3, y = 1:3)
-  expect_named(summarise_at(df, vars(x:y), funs(mean = mean)), c("x_mean", "y_mean"))
-  expect_named(summarise_at(df, vars(x = x), funs(mean = mean, sd = sd)), c("x_mean", "x_sd"))
 
   expect_named(summarise_at(df, vars(x:y), list(mean = mean)), c("x_mean", "y_mean"))
   expect_named(summarise_at(df, vars(x = x), list(mean = mean, sd = sd)), c("x_mean", "x_sd"))
@@ -105,12 +67,6 @@ test_that("empty selection does not select everything (#2009, #1989)", {
   )
 })
 
-test_that("error is thrown with improper additional arguments", {
-  # error messages by base R, not checked
-  expect_error(mutate_all(mtcars, round, 0, 0))
-  expect_error(mutate_all(mtcars, mean, na.rm = TRUE, na.rm = TRUE))
-})
-
 test_that("predicate can be quoted", {
   expected <- mutate_if(mtcars, is_integerish, mean)
   expect_identical(mutate_if(mtcars, "is_integerish", mean), expected)
@@ -118,18 +74,16 @@ test_that("predicate can be quoted", {
 })
 
 test_that("transmute verbs do not retain original variables", {
-  scoped_lifecycle_silence()
-  expect_named(transmute_all(tibble(x = 1:3, y = 1:3), funs(mean, sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
-  expect_named(transmute_if(tibble(x = 1:3, y = 1:3), is_integer, funs(mean, sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
-  expect_named(transmute_at(tibble(x = 1:3, y = 1:3), vars(x:y), funs(mean, sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
-
   expect_named(transmute_all(tibble(x = 1:3, y = 1:3), list(mean = mean, sd = sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
   expect_named(transmute_if(tibble(x = 1:3, y = 1:3), is_integer, list(mean = mean, sd = sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
   expect_named(transmute_at(tibble(x = 1:3, y = 1:3), vars(x:y), list(mean = mean, sd = sd)), c("x_mean", "y_mean", "x_sd", "y_sd"))
 })
 
 test_that("can rename with vars() (#2594)", {
-  expect_equal(mutate_at(tibble(x = 1:3), vars(y = x), mean), tibble(x = 1:3, y = c(2, 2, 2)))
+  expect_identical(
+    mutate_at(tibble(x = 1:3), vars(y = x), mean),
+    tibble(x = 1:3, y = c(2, 2, 2))
+  )
 })
 
 test_that("selection works with grouped data frames (#2624)", {
@@ -138,9 +92,7 @@ test_that("selection works with grouped data frames (#2624)", {
 })
 
 test_that("at selection works even if not all ops are named (#2634)", {
-  scoped_lifecycle_silence()
   df <- tibble(x = 1, y = 2)
-  expect_identical(mutate_at(df, vars(z = x, y), funs(. + 1)), tibble(x = 1, y = 3, z = 2))
   expect_identical(mutate_at(df, vars(z = x, y), list(~. + 1)), tibble(x = 1, y = 3, z = 2))
 })
 
@@ -148,23 +100,6 @@ test_that("can use a purrr-style lambda", {
   expect_identical(summarise_at(mtcars, vars(1:2), ~ mean(.x)), summarise(mtcars, mpg = mean(mpg), cyl = mean(cyl)))
 })
 
-test_that("mutate_at and transmute_at refuses to mutate a grouping variable (#3351, #3480)", {
-  tbl <- tibble(gr1 = rep(1:2, 4), gr2 = rep(1:2, each = 4), x = 1:8) %>%
-    group_by(gr1)
-
-  expect_error(
-    mutate_at(tbl, vars(gr1), sqrt),
-    "Column `gr1` can't be modified because it's a grouping variable",
-    fixed = TRUE
-  )
-
-  expect_error(
-    transmute_at(tbl, vars(gr1), sqrt),
-    "Column `gr1` can't be modified because it's a grouping variable",
-    fixed = TRUE
-  )
-})
-
 test_that("mutate and transmute variants does not mutate grouping variable (#3351, #3480)", {
   tbl <- tibble(gr1 = rep(1:2, 4), gr2 = rep(1:2, each = 4), x = 1:8) %>%
     group_by(gr1)
@@ -180,15 +115,6 @@ test_that("mutate and transmute variants does not mutate grouping variable (#335
   expect_identical(mutate_at(tbl, vars(-group_cols()), sqrt), res)
 })
 
-test_that("summarise_at refuses to treat grouping variables (#3351, #3480)", {
-  tbl <- tibble(gr1 = rep(1:2, 4), gr2 = rep(1:2, each = 4), x = 1:8) %>%
-    group_by(gr1)
-
-  expect_error(
-    summarise_at(tbl, vars(gr1), mean)
-  )
-})
-
 test_that("summarise variants does not summarise grouping variable (#3351, #3480)", {
   tbl <- tibble(gr1 = rep(1:2, 4), gr2 = rep(1:2, each = 4), x = 1:8) %>%
     group_by(gr1)
@@ -207,115 +133,49 @@ test_that("summarise_at removes grouping variables (#3613)", {
   expect_equal(names(res), c("g", "y"))
 })
 
-# Deprecated ---------------------------------------------------------
-
-test_that("_each() and _all() families agree", {
-  scoped_lifecycle_silence()
-  df <- data.frame(x = 1:3, y = 1:3)
-
-  expect_equal(summarise_each(df, funs(mean)), summarise_all(df, mean))
-  expect_equal(summarise_each(df, funs(mean), x), summarise_at(df, vars(x), mean))
-  expect_equal(summarise_each(df, funs(mean = mean), x), summarise_at(df, vars(x), funs(mean = mean)))
-  expect_equal(summarise_each(df, funs(mean = mean), x:y), summarise_at(df, vars(x:y), funs(mean = mean)))
-  expect_equal(summarise_each(df, funs(mean), x:y), summarise_at(df, vars(x:y), mean))
-  expect_equal(summarise_each(df, funs(mean), z = y), summarise_at(df, vars(z = y), mean))
-
-  expect_equal(summarise_each(df, list(mean)), summarise_all(df, mean))
-  expect_equal(summarise_each(df, list(mean), x), summarise_at(df, vars(x), mean))
-  expect_equal(summarise_each(df, list(mean = mean), x), summarise_at(df, vars(x), list(mean = mean)))
-  expect_equal(summarise_each(df, list(mean = mean), x:y), summarise_at(df, vars(x:y), list(mean = mean)))
-  expect_equal(summarise_each(df, list(mean), x:y), summarise_at(df, vars(x:y), mean))
-  expect_equal(summarise_each(df, list(mean), z = y), summarise_at(df, vars(z = y), mean))
-
-  expect_equal(mutate_each(df, funs(mean)), mutate_all(df, mean))
-  expect_equal(mutate_each(df, funs(mean), x), mutate_at(df, vars(x), mean))
-  expect_equal(mutate_each(df, funs(mean = mean), x), mutate_at(df, vars(x), funs(mean = mean)))
-  expect_equal(mutate_each(df, funs(mean = mean), x:y), mutate_at(df, vars(x:y), funs(mean = mean)))
-  expect_equal(mutate_each(df, funs(mean), x:y), mutate_at(df, vars(x:y), mean))
-  expect_equal(mutate_each(df, funs(mean), z = y), mutate_at(df, vars(z = y), mean))
-
-  expect_equal(mutate_each(df, list(mean)), mutate_all(df, mean))
-  expect_equal(mutate_each(df, list(mean), x), mutate_at(df, vars(x), mean))
-  expect_equal(mutate_each(df, list(mean = mean), x), mutate_at(df, vars(x), list(mean = mean)))
-  expect_equal(mutate_each(df, list(mean = mean), x:y), mutate_at(df, vars(x:y), list(mean = mean)))
-  expect_equal(mutate_each(df, list(mean), x:y), mutate_at(df, vars(x:y), mean))
-  expect_equal(mutate_each(df, list(mean), z = y), mutate_at(df, vars(z = y), mean))
-})
-
 test_that("group_by_(at,all) handle utf-8 names (#3829)", {
-  skip_if(getRversion() <= "3.4.0")
-  withr::with_locale( c(LC_CTYPE = "C"), {
-    name <- "\u4e2d"
-    tbl <- tibble(a = 1) %>%
-      setNames(name)
+  local_non_utf8_encoding()
 
-    res <- group_by_all(tbl) %>% groups()
-    expect_equal(res[[1]], sym(name))
+  name <- get_native_lang_string()
+  tbl <- tibble(a = 1) %>%
+    setNames(name)
 
-    res <- group_by_at(tbl, name) %>% groups()
-    expect_equal(res[[1]], sym(name))
-  })
+  res <- group_by_all(tbl) %>% groups()
+  expect_equal(res[[1]], sym(name))
+
+  res <- group_by_at(tbl, name) %>% groups()
+  expect_equal(res[[1]], sym(name))
 })
 
 test_that("*_(all,at) handle utf-8 names (#2967)", {
-  skip_if(getRversion() <= "3.4.0")
-  scoped_lifecycle_silence()
-  withr::with_locale( c(LC_CTYPE = "C"), {
-    name <- "\u4e2d"
-    tbl <- tibble(a = 1) %>%
-      setNames(name)
+  local_non_utf8_encoding()
 
-    res <- tbl %>%
-      mutate_all(funs(as.character)) %>%
-      names()
-    expect_equal(res, name)
+  name <- get_native_lang_string()
+  tbl <- tibble(a = 1) %>%
+    setNames(name)
 
-    res <- tbl %>%
-      mutate_at(name, funs(as.character)) %>%
-      names()
-    expect_equal(res, name)
+  res <- tbl %>%
+    mutate_all(list(as.character)) %>%
+    names()
+  expect_equal(res, name)
 
-    res <- tbl %>%
-      summarise_all(funs(as.character)) %>%
-      names()
-    expect_equal(res, name)
+  res <- tbl %>%
+    mutate_at(name, list(as.character)) %>%
+    names()
+  expect_equal(res, name)
 
-    res <- tbl %>%
-      summarise_at(name, funs(as.character)) %>%
-      names()
-    expect_equal(res, name)
+  res <- tbl %>%
+    summarise_all(list(as.character)) %>%
+    names()
+  expect_equal(res, name)
 
-    res <- tbl %>%
-      mutate_all(list(as.character)) %>%
-      names()
-    expect_equal(res, name)
+  res <- tbl %>%
+    summarise_at(name, list(as.character)) %>%
+    names()
+  expect_equal(res, name)
 
-    res <- tbl %>%
-      mutate_at(name, list(as.character)) %>%
-      names()
-    expect_equal(res, name)
-
-    res <- tbl %>%
-      summarise_all(list(as.character)) %>%
-      names()
-    expect_equal(res, name)
-
-    res <- tbl %>%
-      summarise_at(name, list(as.character)) %>%
-      names()
-    expect_equal(res, name)
-
-    res <- select_at(tbl, name) %>% names()
-    expect_equal(res, name)
-  })
-})
-
-test_that("mutate_all() handles non syntactic names (#4094)", {
-  skip("for now, will fix this after 0.8.0")
-  tbl <- tibble(`..1` = "a")
-  res <- mutate_all(tbl, toupper)
-  expect_equal(names(tbl), names(res))
-  expect_equal(res[["..1"]], "A")
+  res <- select_at(tbl, name) %>% names()
+  expect_equal(res, name)
 })
 
 test_that("summarise_at with multiple columns AND unnamed functions works (#4119)", {
@@ -426,17 +286,6 @@ test_that("colwise mutate handles formulas with constants (#4374)", {
   )
 })
 
-test_that("colwise mutate gives correct error message if column not found (#4374)", {
-  expect_error(
-    mutate_at(tibble(), "test", ~ 1),
-    "column.*test"
-    # either: "Unknown column `test`"
-    #     or: "Can't subset columns that don't exist.\n\033[31mx\033[39m The column `test` doesn't exist."
-    #
-    # depending on which tidyselect is installed
-  )
-})
-
 test_that("colwise mutate handle named chr vectors", {
   res <- tibble(x = 1:10) %>%
     mutate_at(c(y = "x"), mean)
@@ -444,17 +293,63 @@ test_that("colwise mutate handle named chr vectors", {
 })
 
 test_that("colwise verbs soft deprecate quosures (#4330)", {
-  with_lifecycle_errors({
-    expect_error(
-      mutate_at(mtcars, vars(mpg), quo(mean(.)))
+  expect_warning(mutate_at(mtcars, vars(mpg), quo(mean(.))), "quosure")
+  expect_warning(summarise_at(mtcars, vars(mpg), quo(mean(.))), "quosure")
+})
+
+
+test_that("rlang lambda inherit from the data mask (#3843)", {
+  res <- iris %>%
+    mutate_at(
+      vars(starts_with("Petal")),
+      ~ ifelse(Species == "setosa" & . < 1.5, NA, .)
     )
-    expect_error(
-      summarise_at(mtcars, vars(mpg), quo(mean(.)))
+  expected <- iris %>%
+    mutate(
+      Petal.Length = ifelse(Species == "setosa" & Petal.Length < 1.5, NA, Petal.Length),
+      Petal.Width  = ifelse(Species == "setosa" & Petal.Width  < 1.5, NA, Petal.Width)
+    )
+  expect_identical(res, expected)
+
+  res <- iris %>%
+    group_by(Species) %>%
+    mutate_at(
+      vars(starts_with("Petal")),
+      ~ ifelse(Species == "setosa" & . < 1.5, NA, .)
     )
+  expected <- iris %>%
+    group_by(Species) %>%
+    mutate(
+      Petal.Length = ifelse(Species == "setosa" & Petal.Length < 1.5, NA, Petal.Length),
+      Petal.Width  = ifelse(Species == "setosa" & Petal.Width  < 1.5, NA, Petal.Width)
+    )
+  expect_identical(res, expected)
+})
+
+test_that("_if isn't tripped up by columns named 'i' (#5330)", {
+  test_df <- tibble(i = c("a", "b"), j = c(1, 2))
+
+  result_df <- test_df %>%
+    mutate_if(is.character, as.factor)
+
+  expect_equal(result_df$i, as.factor(test_df$i))
+  expect_equal(result_df$j, test_df$j)
+})
+
+# Errors --------------------------------------------
+
+test_that("colwise mutate gives meaningful error messages", {
+  verify_output(test_path("test-colwise-mutate-errors.txt"), {
+    "# column not found"
+    mutate_at(tibble(), "test", ~ 1)
+
+    "# not summarising grouping variables"
+    tbl <- tibble(gr1 = rep(1:2, 4), gr2 = rep(1:2, each = 4), x = 1:8)
+    tbl <- group_by(tbl, gr1)
+    summarise_at(tbl, vars(gr1), mean)
+
+    "# improper additional arguments"
+    mutate_all(mtcars, length, 0, 0)
+    mutate_all(mtcars, mean, na.rm = TRUE, na.rm = TRUE)
   })
-
-  expect_equal(
-    transmute_at(mtcars, vars(mpg), ~. > mean(.)),
-    transmute_at(mtcars, vars(mpg), quo(. > mean(.)))
-  )
 })
