@@ -2,36 +2,38 @@ context("Group by")
 
 df <- data.frame(x = rep(1:3, each = 10), y = rep(1:6, each = 5))
 
-test_that("group_by with add = TRUE adds groups", {
-  add_groups1 <- function(tbl) group_by(tbl, x, y, add = TRUE)
-  add_groups2 <- function(tbl) group_by(group_by(tbl, x, add = TRUE), y, add = TRUE)
+test_that("group_by with .add = TRUE adds groups", {
+  add_groups1 <- function(tbl) group_by(tbl, x, y, .add = TRUE)
+  add_groups2 <- function(tbl) group_by(group_by(tbl, x, .add = TRUE), y, .add = TRUE)
 
-  expect_groups(add_groups1(df), c("x", "y"))
-  expect_groups(add_groups2(df), c("x", "y"))
+  expect_equal(group_vars(add_groups1(df)), c("x", "y"))
+  expect_equal(group_vars(add_groups2(df)), c("x", "y"))
 })
 
-test_that("group_by_ backwards compatibility with add = TRUE adds groups", {
-  scoped_lifecycle_silence()
-  add_groups_extendedclass <- function(tbl) {
-    grouped <- group_by(tbl, x)
-    group_by.default(grouped, y, add = TRUE)
-  }
+test_that("add = TRUE is deprecated", {
+  rlang::scoped_options(lifecycle_verbosity = "warning")
 
-  expect_groups(add_groups_extendedclass(df), c("x", "y"))
+  df <- tibble(x = 1, y = 2)
+
+  expect_warning(
+    out <- df %>% group_by(x) %>% group_by(y, add = TRUE),
+    "deprecated"
+  )
+  expect_equal(group_vars(out), c("x", "y"))
 })
 
 test_that("joins preserve grouping", {
   g <- group_by(df, x)
 
-  expect_groups(inner_join(g, g, by = c("x", "y")), "x")
-  expect_groups(left_join(g, g, by = c("x", "y")), "x")
-  expect_groups(semi_join(g, g, by = c("x", "y")), "x")
-  expect_groups(anti_join(g, g, by = c("x", "y")), "x")
+  expect_equal(group_vars(inner_join(g, g, by = c("x", "y"))), "x")
+  expect_equal(group_vars(left_join(g, g, by = c("x", "y"))), "x")
+  expect_equal(group_vars(semi_join(g, g, by = c("x", "y"))), "x")
+  expect_equal(group_vars(anti_join(g, g, by = c("x", "y"))), "x")
 })
 
 test_that("constructors drops groups", {
   df <- data.frame(x = 1:3) %>% group_by(x)
-  expect_no_groups(tbl_df(df))
+  expect_equal(group_vars(as_tibble(df)), character())
 })
 
 test_that("grouping by constant adds column (#410)", {
@@ -40,6 +42,13 @@ test_that("grouping by constant adds column (#410)", {
   expect_equal(nrow(grouped), 1L)
 })
 
+test_that(".dots is soft deprecated", {
+  rlang::scoped_options(lifecycle_verbosity = "warning")
+
+  df <- tibble(x = 1, y = 1)
+  expect_warning(gf <- group_by(df, .dots = "x"), "deprecated")
+})
+
 # Test full range of variable types --------------------------------------------
 
 
@@ -53,18 +62,18 @@ test_that("local group_by preserves variable types", {
     t = Sys.time() + 1:2,
     c = letters[1:2]
   )
+  attr(df_var$t, "tzone") <- ""
 
   for (var in names(df_var)) {
-    expected <- tibble(unique(df_var[[var]]), n = 1L)
-    names(expected)[1] <- var
+    expected <- tibble(!!var := sort(unique(df_var[[var]])), n = 1L)
 
     summarised <- df_var %>% group_by(!!sym(var)) %>% summarise(n = n())
-    expect_equal(summarised, expected, info = var)
+    expect_equal(summarised, expected)
   }
 })
 
 test_that("mutate does not loose variables (#144)", {
-  df <- tbl_df(data.frame(a = rep(1:4, 2), b = rep(1:4, each = 2), x = runif(8)))
+  df <- tibble(a = rep(1:4, 2), b = rep(1:4, each = 2), x = runif(8))
   by_ab <- group_by(df, a, b)
   by_a <- summarise(by_ab, x = sum(x))
   by_a_quartile <- group_by(by_a, quartile = ntile(x, 4))
@@ -74,26 +83,12 @@ test_that("mutate does not loose variables (#144)", {
 
 test_that("group_by uses shallow copy", {
   m1 <- group_by(mtcars, cyl)
-  expect_no_groups(mtcars)
+  expect_equal(group_vars(mtcars), character())
 
-  expect_equal(dfloc(mtcars), dfloc(m1))
-})
-
-test_that("group_by handles NA in factors #341", {
-  d <- tibble(x = 1:3, f = factor(c("a", "b", NA)))
-  expect_warning(g <- group_by(d, f), "Factor `f` contains implicit NA")
-  expect_equal(group_size(g), rep(1L, 3L))
-
-  d <- tibble(
-    f1 = factor(c(1,1,2,2)),
-    f2 = factor(c(1,2,1,NA)),
-    x  = 1:4
+  expect_equal(
+    lobstr::obj_addrs(mtcars),
+    lobstr::obj_addrs(m1)
   )
-  expect_warning(g <- group_by(d, f1, f2))
-  expect_equal(group_size(g), c(1L,1L,1L,1L))
-
-  expect_warning(g <- group_by(d, f1, f2, .drop = FALSE))
-  expect_equal(group_size(g), c(1L,1L,1L,0L,1L))
 })
 
 test_that("group_by orders by groups. #242", {
@@ -107,43 +102,19 @@ test_that("group_by orders by groups. #242", {
   expect_equal(group_data(df)$a, sqrt(1:10))
 })
 
-test_that("group_by only allows grouping by columns whos class are on the allow list", {
-  df <- data.frame(times = 1:5, x = 1:5)
-  df$times <- as.POSIXlt(seq.Date(Sys.Date(), length.out = 5, by = "day"))
-  expect_error(
-    group_by(df, times),
-    "Column `times` can't be used as a grouping variable because it's a POSIXlt/POSIXt",
-    fixed = TRUE
-  )
+test_that("Can group_by() a POSIXlt", {
+  skip_if_not_installed("tibble", "2.99.99")
+  df <- tibble(x = 1:5, times = as.POSIXlt(seq.Date(Sys.Date(), length.out = 5, by = "day")))
+  g <- group_by(df, times)
+  expect_equal(nrow(group_data(g)), 5L)
 })
 
-test_that("group_by only applies the allow list to grouping variables", {
-  df <- data.frame(times = 1:5, x = 1:5)
-  df$times <- as.POSIXlt(seq.Date(Sys.Date(), length.out = 5, by = "day"))
-
-  res <- group_by(df, x, .drop = FALSE)
-  expect_equal(groups(res), list(sym("x")))
-  expect_identical(
-    group_data(res),
-    structure(tibble(x := 1:5, ".rows" := as.list(1:5)), .drop = FALSE)
-  )
-
-  res <- group_by(df, x)
-  expect_equal(groups(res), list(sym("x")))
-  expect_identical(
-    group_data(res),
-    structure(tibble(x := 1:5, ".rows" := as.list(1:5)), .drop = TRUE)
-  )
-})
-
-test_that("group_by fails when lists are used as grouping variables (#276)", {
-  df <- data.frame(x = 1:3)
-  df$y <- list(1:2, 1:3, 1:4)
-  expect_error(
-    group_by(df, y),
-    "Column `y` can't be used as a grouping variable because it's a list",
-    fixed = TRUE
-  )
+test_that("group_by() handles list as grouping variables", {
+  df <- tibble(x = 1:3, y = list(1:2, 1:3, 1:2))
+  gdata <- group_data(group_by(df, y))
+  expect_equal(nrow(gdata), 2L)
+  expect_equal(gdata$y, list(1:2, 1:3))
+  expect_equal(gdata$.rows, list_of(c(1L, 3L), 2L))
 })
 
 test_that("select(group_by(.)) implicitely adds grouping variables (#170)", {
@@ -151,24 +122,6 @@ test_that("select(group_by(.)) implicitely adds grouping variables (#170)", {
   expect_equal(names(res), c("vs", "mpg"))
 })
 
-test_that("grouped_df errors on NULL labels (#398)", {
-  m <- mtcars %>% group_by(cyl)
-  attr(m, "groups") <- NULL
-  expect_error(
-    m %>% do(mpg = mean(.$mpg)),
-    "is a corrupt grouped_df",
-    fixed = TRUE
-  )
-})
-
-test_that("grouped_df errors on non-existent var (#2330)", {
-  df <- data.frame(x = 1:5)
-  expect_error(
-    grouped_df(df, list(quote(y)), FALSE),
-    "Column `y` is unknown"
-  )
-})
-
 test_that("group_by only creates one group for NA (#401)", {
   x <- as.numeric(c(NA, NA, NA, 10:1, 10:1))
   w <- c(20, 30, 40, 1:10, 1:10) * 10
@@ -189,48 +142,34 @@ test_that("group_by works with zero-row data frames (#486)", {
   df <- data.frame(a = numeric(0), b = numeric(0), g = character(0))
   dfg <- group_by(df, g, .drop = FALSE)
   expect_equal(dim(dfg), c(0, 3))
-  expect_groups(dfg, "g")
+  expect_equal(group_vars(dfg), "g")
   expect_equal(group_size(dfg), integer(0))
 
   x <- summarise(dfg, n = n())
   expect_equal(dim(x), c(0, 2))
-  expect_no_groups(x)
+  expect_equal(group_vars(x), character())
 
   x <- mutate(dfg, c = b + 1)
   expect_equal(dim(x), c(0, 4))
-  expect_groups(x, "g")
+  expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
 
   x <- filter(dfg, a == 100)
   expect_equal(dim(x), c(0, 3))
-  expect_groups(x, "g")
+  expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
 
   x <- arrange(dfg, a, g)
   expect_equal(dim(x), c(0, 3))
-  expect_groups(x, "g")
+  expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
 
   x <- select(dfg, a) # Only select 'a' column; should result in 'g' and 'a'
   expect_equal(dim(x), c(0, 2))
-  expect_groups(x, "g")
+  expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
 })
 
-test_that("grouped_df requires a list of symbols (#665)", {
-  features <- list("feat1", "feat2", "feat3")
-  # error message by assertthat
-  expect_error(grouped_df(data.frame(feat1 = 1, feat2 = 2, feat3 = 3), features))
-})
-
-test_that("group_by gives meaningful message with unknow column (#716)", {
-  expect_error(
-    group_by(iris, wrong_name_of_variable),
-    "Column `wrong_name_of_variable` is unknown",
-    fixed = TRUE
-  )
-})
-
 test_that("[ on grouped_df preserves grouping if subset includes grouping vars", {
   df <- tibble(x = 1:5, ` ` = 6:10)
   by_x <- df %>% group_by(x)
@@ -245,7 +184,7 @@ test_that("[ on grouped_df drops grouping if subset doesn't include grouping var
   by_cyl <- mtcars %>% group_by(cyl)
   no_cyl <- by_cyl %>% `[`(c(1, 3))
 
-  expect_no_groups(no_cyl)
+  expect_equal(group_vars(no_cyl), character())
   expect_is(no_cyl, "tbl_df")
 })
 
@@ -266,33 +205,33 @@ test_that("group_by keeps attributes", {
 })
 
 test_that("ungroup.rowwise_df gives a tbl_df (#936)", {
-  res <- tbl_df(mtcars) %>% rowwise() %>% ungroup() %>% class()
+  res <- mtcars %>% rowwise() %>% ungroup() %>% class()
   expect_equal(res, c("tbl_df", "tbl", "data.frame"))
 })
 
 test_that(paste0("group_by handles encodings for native strings (#1507)"), {
-  with_non_utf8_encoding({
-    special <- get_native_lang_string()
+  local_non_utf8_encoding()
 
-    df <- data.frame(x = 1:3, Eng = 2:4)
+  special <- get_native_lang_string()
 
-    for (names_converter in c(enc2native, enc2utf8)) {
-      for (dots_converter in c(enc2native, enc2utf8)) {
-        names(df) <- names_converter(c(special, "Eng"))
-        res <- group_by(df, !!!syms(dots_converter(special)))
-        expect_equal(names(res), names(df))
-        expect_groups(res, special)
-      }
-    }
+  df <- data.frame(x = 1:3, Eng = 2:4)
 
-    for (names_converter in c(enc2native, enc2utf8)) {
+  for (names_converter in c(enc2native, enc2utf8)) {
+    for (dots_converter in c(enc2native, enc2utf8)) {
       names(df) <- names_converter(c(special, "Eng"))
-
-      res <- group_by(df, !!!special)
-      expect_equal(names(res), c(names(df), deparse(special)))
-      expect_equal(groups(res), list(as.name(enc2native(deparse(special)))))
+      res <- group_by(df, !!!syms(dots_converter(special)))
+      expect_equal(names(res), names(df))
+      expect_equal(group_vars(res), special)
     }
-  })
+  }
+
+  for (names_converter in c(enc2native, enc2utf8)) {
+    names(df) <- names_converter(c(special, "Eng"))
+
+    res <- group_by(df, !!!special)
+    expect_equal(names(res), c(names(df), deparse(special)))
+    expect_equal(groups(res), list(as.name(enc2native(deparse(special)))))
+  }
 })
 
 test_that("group_by handles raw columns (#1803)", {
@@ -324,11 +263,11 @@ test_that("group_by() does not affect input data (#3028)", {
 })
 
 test_that("group_by() does not mutate for nothing when using the .data pronoun (#2752, #3533)", {
-  expect_equal(
+  expect_identical(
     iris %>% group_by(Species) %>% group_by(.data$Species),
     iris %>% group_by(Species)
   )
-  expect_equal(
+  expect_identical(
     iris %>% group_by(Species) %>% group_by(.data[["Species"]]),
     iris %>% group_by(Species)
   )
@@ -345,23 +284,11 @@ test_that("tbl_sum gets the right number of groups", {
   expect_equal(res, c("A tibble" = "4 x 1", "Groups" = "x [2]"))
 })
 
-test_that("grouped data frames support drop=TRUE (#3714)", {
-  expect_is(group_by(iris, Species)[ , "Sepal.Width", drop=TRUE], "numeric")
-
-  expect_is(group_by(iris, Species)[ , c("Species", "Sepal.Width"), drop=TRUE], "grouped_df")
-})
-
 test_that("group_by ignores empty quosures (3780)", {
   empty <- quo()
   expect_equal(group_by(mtcars, cyl), group_by(mtcars, cyl, !!empty))
 })
 
-test_that("group_cols() selects grouping variables", {
-  expect_identical(select(mtcars, group_cols()), select(mtcars))
-  expect_identical(select(group_by(mtcars, cyl, am), group_cols()), group_by_all(select(mtcars, cyl, am)))
-  expect_identical(mutate_at(group_by(iris, Species), vars(-group_cols()), `/`, 100), mutate_all(group_by(iris, Species), `/`, 100))
-})
-
 # Zero groups ---------------------------------------------------
 
 test_that("mutate handles grouped tibble with 0 groups (#3935)", {
@@ -405,7 +332,11 @@ test_that("arrange handles grouped tibble with 0 groups (#3935)", {
 test_that("group_by() with empty spec produces a grouped data frame with 0 grouping variables", {
   gdata <- group_data(group_by(iris))
   expect_equal(names(gdata), ".rows")
-  expect_equal(gdata$.rows, list(1:nrow(iris)))
+  expect_equal(gdata$.rows, list_of(1:nrow(iris)))
+
+  gdata <- group_data(group_by(iris, !!!list()))
+  expect_equal(names(gdata), ".rows")
+  expect_equal(gdata$.rows, list_of(1:nrow(iris)))
 })
 
 # .drop = TRUE ---------------------------------------------------
@@ -418,7 +349,7 @@ test_that("group_by(.drop = TRUE) drops empty groups (4061)", {
   expect_identical(
     group_data(res),
     structure(
-      tibble(Species = factor("setosa", levels = levels(iris$Species)), .rows := list(1:50)),
+      tibble(Species = factor("setosa", levels = levels(iris$Species)), .rows := list_of(1:50)),
       .drop = TRUE
     )
   )
@@ -456,6 +387,15 @@ test_that("grouped data frames remember their .drop = FALSE (#4337)", {
   expect_false(group_by_drop_default(res2))
 })
 
+test_that("group_by(.drop = FALSE) preserve ordered factors (#5455)", {
+  df <- tibble(x = ordered("x"))
+  drop <- df %>% group_by(x) %>% group_data()
+  nodrop <- df %>% group_by(x, .drop = FALSE) %>% group_data()
+
+  expect_equal(is.ordered(drop$x), is.ordered(nodrop$x))
+  expect_true(is.ordered(nodrop$x))
+})
+
 test_that("summarise maintains the .drop attribute (#4061)", {
   df <- tibble(
     f1 = factor("a", levels = c("a", "b", "c")),
@@ -501,18 +441,11 @@ test_that("group_by(add = TRUE) sets .drop if the origonal data was .drop", {
     x  = 48
   )
 
-  res <- group_by(group_by(d, f1, .drop = TRUE), f2, add = TRUE)
+  res <- group_by(group_by(d, f1, .drop = TRUE), f2, .add = TRUE)
   expect_equal(n_groups(res), 1L)
   expect_true(group_by_drop_default(res))
 })
 
-test_that("group_by() makes a shallow copy of data even in the corner case", {
-  df <- data.frame(x = 1:4)
-  gdf <- group_by(df)
-  expect_true(inherits(gdf, "tbl_df"))
-  expect_false(inherits(df, "tbl_df"))
-})
-
 test_that("group_by_drop_default() is forgiving about corrupt grouped df (#4306)",{
   df <- tibble(x = 1:2, y = 1:2) %>%
     structure(class = c("grouped_df", "tbl_df", "tbl", "data.frame"))
@@ -525,5 +458,82 @@ test_that("group_by() puts NA groups last in STRSXP (#4227)", {
     group_by(x) %>%
     group_data()
   expect_identical(res$x, c("apple", "banana", NA_character_))
-  expect_identical(res$.rows, list(1L, 3L, 2L))
+  expect_identical(res$.rows, list_of(1L, 3L, 2L))
 })
+
+test_that("group_by() does not create arbitrary NA groups for factors when drop = TRUE (#4460)", {
+  res <- expect_warning(group_data(group_by(iris, Species)[0, ]), NA)
+  expect_equal(nrow(res), 0L)
+
+  res <- expect_warning(group_data(group_by(iris[0, ], Species)), NA)
+  expect_equal(nrow(res), 0L)
+})
+
+test_that("group_by() can handle auto splicing in the mutate() step", {
+  expect_identical(
+    iris %>% group_by(Species),
+    iris %>% group_by(data.frame(Species = Species))
+  )
+
+  expect_identical(
+    iris %>% group_by(Species),
+    iris %>% group_by(across(Species))
+  )
+
+  expect_identical(
+    iris %>% mutate(across(starts_with("Sepal"), round)) %>% group_by(Sepal.Length, Sepal.Width),
+    iris %>% group_by(across(starts_with("Sepal"), round))
+  )
+
+  df <- tibble(x = c(1, 2), y = c(1, 2))
+  expect_identical(df %>% group_by(across(character())), df)
+  expect_identical(df %>% group_by(across(NULL)), df)
+
+  expect_identical(df %>% group_by(x) %>% group_by(across(character())), df)
+  expect_identical(df %>% group_by(x) %>% group_by(across(NULL)), df)
+})
+
+test_that("group_by() can combine usual spec and auto-splicing-mutate() step", {
+  expect_identical(
+    iris %>% mutate(across(starts_with("Sepal"), round)) %>% group_by(Sepal.Length, Sepal.Width, Species),
+    iris %>% group_by(across(starts_with("Sepal"), round), Species)
+  )
+
+  expect_identical(
+    iris %>% mutate(across(starts_with("Sepal"), round)) %>% group_by(Species, Sepal.Length, Sepal.Width),
+    iris %>% group_by(Species, across(starts_with("Sepal"), round))
+  )
+})
+
+# mutate() semantics
+
+test_that("group_by() has mutate() semantics (#4984)", {
+  expect_equal(
+    tibble(a = 1, b = 2) %>% group_by(c = a * b, d = c + 1),
+    tibble(a = 1, b = 2) %>% mutate(c = a * b, d = c + 1) %>% group_by(c, d)
+  )
+})
+
+test_that("implicit mutate() operates on ungrouped data (#5598)", {
+  vars <- tibble(x = c(1,2), y = c(3,4), z = c(5,6)) %>%
+    dplyr::group_by(y) %>%
+    dplyr::group_by(across(any_of(c('y','z')))) %>%
+    dplyr::group_vars()
+  expect_equal(vars, c("y", "z"))
+})
+
+# Errors ------------------------------------------------------------------
+
+test_that("group_by() and ungroup() give meaningful error messages", {
+  verify_output(test_path("test-group-by-errors.txt"), {
+    df <- tibble(x = 1, y = 2)
+
+    df %>% group_by(unknown)
+
+    df %>% ungroup(x)
+    df %>% group_by(x, y) %>% ungroup(z)
+
+    df %>% group_by(z = a + 1)
+  })
+})
+
